@@ -1,16 +1,17 @@
 import random
+from typing import Callable
 
 
-def backoff_linear_with_jitter(wait_time, jitter):
-    def func(attempt):  # pylint: disable=unused-argument
+def backoff_linear_with_jitter(wait_time: float, jitter: float) -> Callable[[int], float]:
+    def func(attempt: int) -> float:  # pylint: disable=unused-argument
         multiplier = jitter * (random.random() * 2 - 1)
         return wait_time * (1 + multiplier)
 
     return func
 
 
-def backoff_exponential_with_jitter(base, cap):
-    def func(attempt):
+def backoff_exponential_with_jitter(base: float, cap: float) -> Callable[[int], float]:
+    def func(attempt: int) -> float:  # pylint: disable=unused-argument
         try:
             res = (2**attempt) * base * random.random()
         except OverflowError:
@@ -24,8 +25,8 @@ def backoff_exponential_with_jitter(base, cap):
     return func
 
 
-def backoff_exponential_jittered_min_interval(base=0.05, cap=60):
-    def func(attempt):
+def backoff_exponential_jittered_min_interval(base: float = 0.05, cap: float = 60) -> Callable[[int], float]:
+    def func(attempt: int) -> float:  # pylint: disable=unused-argument
         try:
             base_interval = (2**attempt) * base
             res = base_interval / 2 + base_interval * random.random()
@@ -40,5 +41,5 @@ def backoff_exponential_jittered_min_interval(base=0.05, cap=60):
     return func
 
 
-def default_backoff():
+def default_backoff() -> Callable[[int], float]:
     return backoff_exponential_with_jitter(0.05, 60)
