@@ -1,10 +1,10 @@
-# -*- coding: utf-8 -*-
-
+# pylint: disable=no-member
+# mypy: ignore-errors
 import logging
 import random
+from typing import Iterable, NamedTuple
 
 from google.protobuf.field_mask_pb2 import FieldMask
-from six import string_types
 
 import yandex.cloud.dataproc.v1.cluster_pb2 as cluster_pb
 import yandex.cloud.dataproc.v1.cluster_service_pb2 as cluster_service_pb
@@ -18,7 +18,20 @@ import yandex.cloud.dataproc.v1.subcluster_service_pb2 as subcluster_service_pb
 import yandex.cloud.dataproc.v1.subcluster_service_pb2_grpc as subcluster_service_grpc_pb
 
 
-class Dataproc(object):
+class InitializationAction(NamedTuple):
+    uri: str  # Uri of the executable file
+    args: Iterable[str]  # Arguments to the initialization action
+    timeout: int  # Execution timeout
+
+    def to_grpc(self):
+        return cluster_pb.InitializationAction(
+            uri=self.uri,
+            args=self.args,
+            timeout=self.timeout,
+        )
+
+
+class Dataproc:
     """
     A base hook for Yandex.Cloud Data Proc.
 
@@ -39,12 +52,13 @@ class Dataproc(object):
             self.log = logging.getLogger()
             self.log.addHandler(logging.NullHandler())
         self.cluster_id = None
+        self.subnet_id = None
         self.default_folder_id = default_folder_id
         self.default_public_ssh_key = default_public_ssh_key
 
     def create_cluster(
         self,
-        s3_bucket,
+        s3_bucket=None,
         folder_id=None,
         cluster_name=None,
         cluster_description="",
@@ -54,6 +68,7 @@ class Dataproc(object):
         services=None,
         zone="ru-central1-b",
         service_account_id=None,
+        environment=None,
         masternode_resource_preset=None,
         masternode_disk_size=None,
         masternode_disk_type=None,
@@ -73,6 +88,14 @@ class Dataproc(object):
         computenode_cpu_utilization_target=None,
         computenode_decommission_timeout=None,
         log_group_id=None,
+        properties=None,
+        enable_ui_proxy=False,
+        host_group_ids=None,
+        security_group_ids=None,
+        initialization_actions=None,
+        oslogin_enabled=False,
+        labels=None,
+        autoscaling_service_account_id=None,
     ):
         """
         Create Yandex.Cloud Data Proc cluster.
@@ -101,6 +124,8 @@ class Dataproc(object):
         :param service_account_id: Service account id for the cluster.
                                    Service account can be created inside the folder.
         :type service_account_id: str
+        :param environment: Environment for the cluster. Possible options: PRODUCTION, PRESTABLE.
+        :type environment: str
         :param masternode_resource_preset: Resources preset (CPU+RAM configuration)
                                            for the master node of the cluster.
         :type masternode_resource_preset: str
@@ -149,6 +174,25 @@ class Dataproc(object):
         :param log_group_id: Id of log group to write logs. By default logs will be sent to default log group.
                              To disable cloud log sending set cluster property dataproc:disable_cloud_logging = true
         :type log_group_id: str
+        :param properties: Properties passed to main node software, in key you need to use prefix: 'hdfs:dfs.hosts'
+        :type properties: Dict[str, str]
+        :param enable_ui_proxy: Enable UI Proxy feature for forwarding Hadoop components web interfaces
+                                Docs: https://cloud.yandex.com/en-ru/docs/data-proc/concepts/ui-proxy
+        :type enable_ui_proxy: Bool
+        :param host_group_ids: Dedicated host groups to place VMs of cluster on.
+                               Docs: https://cloud.yandex.com/en-ru/docs/compute/concepts/dedicated-host
+        :type host_group_ids: Iterable[str] | None
+        :param security_group_ids: User security groups
+                                   Docs: https://cloud.yandex.com/en-ru/docs/data-proc/concepts/network#security-groups
+        :type security_group_ids: Iterable[str] | None
+        :param initialization_actions: Set of init-actions to run when cluster starts
+        :type initialization_actions: Iterable[InitializationAction] | None
+        :param oslogin_enabled: Enable authorization via OS Login for cluster.
+        :type oslogin_enabled: bool
+        :param labels: Cluster labels as key:value pairs. No more than 64 per resource.
+        :type labels: Dict[str, str]
+        :param autoscaling_service_account_id: Service account to be used by the Instance Groups service.
+        :type autoscaling_service_account_id: str
 
         :return: Cluster ID
         :rtype: str
@@ -164,7 +208,7 @@ class Dataproc(object):
 
         if not cluster_name:
             random_int = random.randint(0, 999)
-            cluster_name = "dataproc-{random_int}".format(random_int=random_int)
+            cluster_name = f"dataproc-{random_int}"
 
         if not subnet_id:
             network_id = self.sdk.helpers.find_network_id(folder_id)
@@ -173,17 +217,14 @@ class Dataproc(object):
         if not service_account_id:
             service_account_id = self.sdk.helpers.find_service_account_id(folder_id)
 
-        if not ssh_public_keys:
+        if not ssh_public_keys and not oslogin_enabled:
             if self.default_public_ssh_key:
                 ssh_public_keys = (self.default_public_ssh_key,)
             else:
-                raise RuntimeError("Public ssh keys must be specified.")
-        elif isinstance(ssh_public_keys, string_types):
+                raise RuntimeError("Public ssh keys must be specified or OS Login must be enabled.")
+        elif isinstance(ssh_public_keys, str):
             ssh_public_keys = [ssh_public_keys]
 
-        if not s3_bucket:
-            raise RuntimeError("Object storage (S3) bucket must be specified.")
-
         gib = 1024**3
         if masternode_disk_size:
             masternode_disk_size *= gib
@@ -202,19 +243,22 @@ class Dataproc(object):
                 ),
                 subnet_id=subnet_id,
                 hosts_count=1,
-            ),
-            cluster_service_pb.CreateSubclusterConfigSpec(
-                name="data",
-                role=subcluster_pb.Role.DATANODE,
-                resources=common_pb.Resources(
-                    resource_preset_id=datanode_resource_preset,
-                    disk_size=datanode_disk_size,
-                    disk_type_id=datanode_disk_type,
-                ),
-                subnet_id=subnet_id,
-                hosts_count=datanode_count,
-            ),
+            )
         ]
+        if datanode_count:
+            subclusters.append(
+                cluster_service_pb.CreateSubclusterConfigSpec(
+                    name="data",
+                    role=subcluster_pb.Role.DATANODE,
+                    resources=common_pb.Resources(
+                        resource_preset_id=datanode_resource_preset,
+                        disk_size=datanode_disk_size,
+                        disk_type_id=datanode_disk_type,
+                    ),
+                    subnet_id=subnet_id,
+                    hosts_count=datanode_count,
+                )
+            )
 
         if computenode_count:
             autoscaling_config = None
@@ -252,13 +296,26 @@ class Dataproc(object):
                 hadoop=cluster_pb.HadoopConfig(
                     services=services,
                     ssh_public_keys=ssh_public_keys,
+                    properties=properties,
+                    initialization_actions=(
+                        (initialization_actions and [action.to_grpc() for action in initialization_actions])
+                        if initialization_actions
+                        else None
+                    ),
+                    oslogin_enabled=oslogin_enabled,
                 ),
                 subclusters_spec=subclusters,
             ),
             zone_id=zone,
             service_account_id=service_account_id,
+            environment=environment,
             bucket=s3_bucket,
+            ui_proxy=enable_ui_proxy,
+            host_group_ids=host_group_ids,
+            security_group_ids=security_group_ids,
             log_group_id=log_group_id,
+            labels=labels,
+            autoscaling_service_account_id=autoscaling_service_account_id,
         )
         result = self.sdk.create_operation_and_get_result(
             request,
@@ -353,7 +410,7 @@ class Dataproc(object):
             disk_type_id=disk_type,
         )
 
-        self.log.info("Adding subcluster to cluster {cluster_id}".format(cluster_id=cluster_id))
+        self.log.info("Adding subcluster to cluster %s", cluster_id)
         autoscaling_config = None
         if max_hosts_count:
             autoscaling_config = subcluster_pb.AutoscalingConfig(
@@ -395,7 +452,7 @@ class Dataproc(object):
         if not cluster_id:
             raise RuntimeError("Cluster id must be specified.")
 
-        self.log.info("Updating cluster {cluster_id}".format(cluster_id=cluster_id))
+        self.log.info("Updating cluster %s", cluster_id)
         mask = FieldMask(paths=["description"])
         request = cluster_service_pb.UpdateClusterRequest(
             cluster_id=cluster_id,
@@ -420,7 +477,7 @@ class Dataproc(object):
         if not cluster_id:
             raise RuntimeError("Cluster id must be specified.")
 
-        self.log.info("Deleting cluster {cluster_id}".format(cluster_id=cluster_id))
+        self.log.info("Deleting cluster %s", cluster_id)
         request = cluster_service_pb.DeleteClusterRequest(cluster_id=cluster_id)
         return self.sdk.create_operation_and_get_result(
             request,
@@ -429,6 +486,45 @@ class Dataproc(object):
             meta_type=cluster_service_pb.DeleteClusterMetadata,
         )
 
+    def stop_cluster(self, cluster_id=None, decommission_timeout=0):
+        """
+        Stop Yandex.Cloud Data Proc cluster.
+        :param cluster_id: ID of the cluster to stop.
+        :type cluster_id: str
+        :param decommission_timeout: Timeout to gracefully decommission nodes during downscaling. In seconds.
+        :type decommission_timeout: int
+        """
+        cluster_id = cluster_id or self.cluster_id
+        if not cluster_id:
+            raise RuntimeError("Cluster id must be specified.")
+
+        request = cluster_service_pb.StopClusterRequest(
+            cluster_id=cluster_id, decommission_timeout=decommission_timeout
+        )
+        return self.sdk.create_operation_and_get_result(
+            request,
+            service=cluster_service_grpc_pb.ClusterServiceStub,
+            method_name="Stop",
+            meta_type=cluster_service_pb.StopClusterMetadata,
+        )
+
+    def start_cluster(self, cluster_id=None):
+        """
+        Start Yandex.Cloud Data Proc cluster.
+        :param cluster_id: ID of the cluster to start.
+        :type cluster_id: str
+        """
+        cluster_id = cluster_id or self.cluster_id
+        if not cluster_id:
+            raise RuntimeError("Cluster id must be specified.")
+        request = cluster_service_pb.StartClusterRequest(cluster_id=cluster_id)
+        return self.sdk.create_operation_and_get_result(
+            request,
+            service=cluster_service_grpc_pb.ClusterServiceStub,
+            method_name="Start",
+            meta_type=cluster_service_pb.StartClusterMetadata,
+        )
+
     def create_hive_job(
         self,
         query=None,
@@ -463,7 +559,7 @@ class Dataproc(object):
             raise RuntimeError("Cluster id must be specified.")
         if (query and query_file_uri) or not (query or query_file_uri):
             raise RuntimeError("Either query or query_file_uri must be specified.")
-        self.log.info("Running Hive job. Cluster ID: {cluster_id}".format(cluster_id=cluster_id))
+        self.log.info("Running Hive job. Cluster ID: %s", cluster_id)
 
         hive_job = job_pb.HiveJob(
             query_file_uri=query_file_uri,
@@ -530,7 +626,7 @@ class Dataproc(object):
         cluster_id = cluster_id or self.cluster_id
         if not cluster_id:
             raise RuntimeError("Cluster id must be specified.")
-        self.log.info("Running Mapreduce job. Cluster ID: {cluster_id}".format(cluster_id=cluster_id))
+        self.log.info("Running Mapreduce job. Cluster ID: %s", cluster_id)
 
         request = job_service_pb.CreateJobRequest(
             cluster_id=cluster_id,
@@ -602,7 +698,7 @@ class Dataproc(object):
         cluster_id = cluster_id or self.cluster_id
         if not cluster_id:
             raise RuntimeError("Cluster id must be specified.")
-        self.log.info("Running Spark job. Cluster ID: {cluster_id}".format(cluster_id=cluster_id))
+        self.log.info("Running Spark job. Cluster ID: %s", cluster_id)
 
         request = job_service_pb.CreateJobRequest(
             cluster_id=cluster_id,
@@ -677,7 +773,7 @@ class Dataproc(object):
         cluster_id = cluster_id or self.cluster_id
         if not cluster_id:
             raise RuntimeError("Cluster id must be specified.")
-        self.log.info("Running Pyspark job. Cluster ID: {cluster_id}".format(cluster_id=cluster_id))
+        self.log.info("Running Pyspark job. Cluster ID: %s", cluster_id)
         request = job_service_pb.CreateJobRequest(
             cluster_id=cluster_id,
             name=name,
